@@ -1118,3 +1118,24 @@ def test_same_thread_assets_independence(catalog, folder):
     print(html2)
     assert html1 == Markup(expected_1)
     assert html2 == Markup(expected_2)
+
+
+def test_thread_safety_of_template_globals(catalog, folder):
+    NUM_THREADS = 5
+    (folder / "Page.jinja").write_text("{{ globalvar if globalvar is defined else 'not set' }}")
+
+    def render(i):
+        return catalog.render("Page", __globals={"globalvar": i})
+
+    threads = []
+
+    for i in range(NUM_THREADS):
+        thread = ThreadWithReturnValue(target=render, args=(i,))
+        threads.append(thread)
+        thread.start()
+
+    results = [thread.join() for thread in threads]
+
+    for i, result in enumerate(results):
+        assert result == Markup(str(i))
+
